'  9-bit PCM Player, plays stereo PCM (up to 16-bit) from a circular buffer
'  Copyright 2009 David Sloan
'  See end of file for terms of use.

VAR
  long  dacpars[4]
  long  smppars[6]   
  long  SampleVarL
  long  SampleVarR
  byte  running
  byte  cogs[2]

Pub start(Lpin, Rpin, pBuffer, BufferLen, pBufferPos, samplePeriod) 
  dacpars[0] := Lpin
  dacpars[1] := Rpin
  dacpars[2] := @SampleVarL
  dacpars[3] := @SampleVarR

  smppars[0] := @SampleVarL
  smppars[1] := @SampleVarR
  smppars[2] := pBuffer
  smppars[3] := BufferLen
  smppars[4] := pBufferPos
  smppars[5] := samplePeriod
  if running <> 1
    cogs[0] := cognew(@dac, @dacpars)
    cogs[1] := cognew(@smp, @smppars)
    running := 1 

pub stop
  if running == 1
    cogstop(cogs[0])
    cogstop(cogs[1])
    running := 0
    

DAT
org 0
              'retrieve parameters
dac           mov       time, par               'using time as temp var
              rdlong    dacLpin, time
              add       time, #4
              rdlong    dacRpin, time
              add       time, #4
              rdlong    pLeft, time
              add       time, #4
              rdlong    pRight, time
              'set pin directions
              mov       time, #1
              shl       time, dacLpin
              or        dira, time
              mov       time, #1
              shl       time, dacRpin
              or        dira, time
              'set ctr state
              mov       ctra, ctrval
              movs      ctra, dacLpin
              mov       frqa, #1       'set counter to increment 1 each cycle
              mov       ctrb, ctrval
              movs      ctrb, dacRpin
              mov       frqb, #1
              'set up wait
              mov       time, cnt      'record current time
              add       time, period   'establish next period

              
loop          rdlong    Lvalue, pLeft   
              and       Lvalue, ValMask
              rdlong    Rvalue, pRight
              and       Rvalue, ValMask
              waitcnt   time, period     'wait until next period
              neg       phsa, Lvalue  
              neg       phsb, Rvalue
              jmp       #loop            'loop for next cycle       

dacLpin       long 0
dacRpin       long 0
pLeft         long 0
pRight        long 0
ctrval        long %00100 << 26 + 0 
period        long 512               '~156kHz period (_clkfreq / period)
ValMask       long %1_1111_1111
time          res 1
Lvalue        res 1
Rvalue        res 1

org 0
              'retrieve parameters
smp           mov       tmp, par
              rdlong    pL, tmp
              add       tmp, #4
              rdlong    pR, tmp
              add       tmp, #4
              rdlong    pbuf, tmp
              add       tmp, #4
              rdlong    buflen, tmp
              add       tmp, #4
              rdlong    pbufPos, tmp
              add       tmp, #4
              rdlong    sPeriod, tmp
              wrlong    Zero, pbufpos
              shl       buflen, #2

              mov       bufpos, #0
              
              mov       sTime, cnt
              add       sTime, sPeriod

sloop         mov       tmp, pbuf
              add       tmp, BufPos
              rdlong    sValL, tmp
              waitcnt   sTime, sPeriod
              mov       sValR, SValL
              and       sValL, WordMask
              sub       sValR, halfword
              sub       sValL, halfword
              and       sValR, WordMask
              and       sValL, WordMask
              shr       sValR, #23
              shr       sValL, #7
              wrlong    sValR, pL
              add       BufPos, #4
              wrlong    BufPos, PBufPos
              cmp       BufPos, buflen          wc, wz
        if_ae mov       bufpos, #0
              wrLong    sValL, pR
              jmp       #sLoop

Zero          long 0
WordMask      long $0000_ffff
HalfWord      long $7fff

pL            res 1
pR            res 1
pbuf          res 1
bufLen        res 1
pBufPos       res 1
bufpos        res 1
sPeriod       res 1
sTime         res 1
sValL         res 1
sValR         res 1
tmp           res 1


{{
                            TERMS OF USE: MIT License

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
}}      